;LED 3-DIGIT COUNTER - Version 1
;BRILLIANT LED PROJECTS - Nick Dossis - McGraw Hill 2011
;Multiplexed 7-segment display using outputs B0-B7 & A0-A3 
;Push button input on A4 increments the counter
;Holding A4 down resets the counter

#DEFINE BANK0 bcf STATUS,5
#DEFINE BANK1 bsf STATUS,5

           list p=16f628       ; 
                               ; 

           include p16f628.inc ; 

           __config  $3F30     ; internal 4MHz oscillator

	CBLOCK h'20'	       ;
DISPL       
CLOCK
SPEED
FLAG
UNITS
TENS
HUNDRED
RESET
	ENDC	               ;

           ORG 0               ;
           goto BEGIN
           ORG 4               ;
           goto BEGIN
           ORG 5               ;
           goto BEGIN

;7-Segment Display Character set

ALPHA:  addwf PCL,F        ;LEd start-up word character table
        retlw B'10110100' ;L
        retlw B'10111110' ;E
        retlw B'11111001' ;d
        

CODE:   addwf PCL,F       ;7-segment display character table
        retlw B'01110111' ;0
        retlw B'01000001' ;1
        retlw B'00111011' ;2
        retlw B'01101011' ;3
        retlw B'01001101' ;4
        retlw B'01101110' ;5
        retlw B'01111100' ;6
        retlw B'01000011' ;7
        retlw B'01111111' ;8
        retlw B'01101111' ;9

;Start of program

BEGIN:	movlw 7	          ;configure Port A as a digital port
        movwf CMCON     	

	BANK1	
        movlw B'00010000' ;port a and b as outputs, except A4 is an input
	movwf TRISA      
        movlw B'00000000'
	movwf TRISB      
        movlw B'10000001' ;TMR0 rate = 1:4 
        movwf OPTION_REG  ;bit 7 is high = pull-ups off
	BANK0	          
	
        movlw 5	          ;
	movwf SPEED	  ;sets the pause speed = 5

        clrf DISPL
	clrf PORTA	  ;clears port A
	clrf PORTB	  ;clears port B
	clrf UNITS	  ;
	clrf TENS	  ;
	clrf HUNDRED	  ;
	clrf FLAG	  ;
	clrf RESET        ;
	
;Start-up message
;D1 Sequence

START:	bsf PORTA,0 	;activate TR1
	movf DISPL,W	;move display variable into w
	call ALPHA	;calls letter coding
	movwf PORTB	;moves the frame value to port B to illuminate D1
        call PAUSE	;pause to stabilize the display
	clrf PORTA	;deactivates TR1
	incf DISPL,F

;D2 Sequence

	bsf PORTA,1 	;activate TR2
	movf DISPL,W	;move display variable into w
	call ALPHA	;calls letter coding
	movwf PORTB	;moves the frame value to port B to illuminate D2
        Call PAUSE      ;pause to stabilize the display
	clrf PORTA	;deactivates TR2
        incf DISPL,F

;D3 Sequence

	bsf PORTA,3 	;activate TR2
	movf DISPL,W	;move display variable into w
	call ALPHA	;calls letter coding
	movwf PORTB	;moves the frame value to port B to illuminate D3
        call PAUSE      ;pause to stabilize the display
	clrf PORTA	;deactivates TR3
        clrf DISPL


INPUT:  btfsc PORTA,4	;checks input on A4, is it high?
	goto MAIN1	;yes, goto main counter routine
	goto START	;no, keep looping around


;Main counter routines

MAIN1:  bsf FLAG,0	;set debounce flag so that the counter starts at 000

;D1 Sequence

MAIN:	bsf PORTA,0 	;activate TR1
	movf HUNDRED,W	;moves hundreds variable into w
	call CODE	;get display code
	movwf PORTB	;moves the frame value to port B to illuminate D1
        call PAUSE	;pause to stabilize the display
	clrf PORTA	;deactivates TR1

;D2 Sequence

	bsf PORTA,1 	;activate TR2
	movf TENS,W	;move tens variable into w
	call CODE	;get display code
	movwf PORTB	;moves the frame value to port B to illuminate D2
        Call PAUSE      ;pause to stabilize the display
	clrf PORTA	;deactivates TR2

;D3 Sequence

	bsf PORTA,3 	;activate TR2
	movf UNITS,W	;move units variable into w
	call CODE	;get display code
	movwf PORTB	;moves the frame value to port B to illuminate D3
        call PAUSE      ;pause to stabilize the display
	clrf PORTA	;deactivates TR3

;checks input A4 for a key press, the flag checks are to debounce the input

KEY:    btfsc PORTA,4	;checks input on A4, is it high?
	call COUNT	;yes, if A4 is pressed/high then call the counter routine to increment digits
	btfss PORTA,4	;check input A4, is it low?
	clrf FLAG	;yes, if A4 is low then clear the flag to allow further counting

	movf RESET,W	;move the reset counter to w
	xorlw 200	;
	btfsc STATUS,Z	;does the reset counter = 200?
	call CLEAR	;yes, reset the display	

	goto MAIN	;loop around

;clock count routine including validity checks for unit and tens values

COUNT:  incf RESET,F	;increment the reset counter
	btfsc FLAG,0	;is flag clear?
	return		;no return to display without incrementing        
	incf UNITS,F	;if input is high and flag is low then increment counter
	movf UNITS,W	;move units to w
	xorlw 10	;
	btfsc STATUS,Z	;does units = 10
	goto TEN	;yes
	bsf FLAG,0	;no, set flag bit for next time
	clrf RESET	;clears the reset counter
	return		;return

TEN:	clrf UNITS	;clear units
	incf TENS,F	;increment tens
	movf TENS,W	;move units to w
	xorlw 10	;
	btfsc STATUS,Z	;does tens = 10
	goto HUND	;yes
	bsf FLAG,0	;no, set flag bit for next time
	return		;

HUND:	clrf UNITS	;clear units
	clrf TENS	;clear tens
	incf HUNDRED,F	;increment hundreds
	movf HUNDRED,W	;move units to w
	xorlw 10	;
	btfsc STATUS,Z	;does hundreds = 10
	goto CLEAR	;yes, clear all digits
	bsf FLAG,0	;no, set flag bit for next time
	return		;

CLEAR:  clrf TENS	;yes, clear tens
	clrf UNITS	;clear units
	clrf HUNDRED	;clear hundreds
	bsf FLAG,0	;no, set flag bit for next time
	clrf RESET	;reset 
	return

;delay routine

PAUSE   movf SPEED,W       ;set pause counter to value of speed
        movwf CLOCK        ; 
        clrf INTCON        ;clear interupt flag
                            
LOOP4   btfss INTCON,2     ;has the intcon timer time-out been detected?
        goto LOOP4         ;no, loop until it is detected
        bcf INTCON,2       ;yes, clear intcon
        decfsz CLOCK,F     ;decrement the clock counter, is it zero?
        goto LOOP4         ;no
        return             ;yes


        .END               ;