;LED LIGHT SWORD - Version 1
;BRILLIANT LED PROJECTS - Nick Dossis - McGraw Hill 2011
;LED outputs B0 - B7
;Port A2 = Power up switch
;Port A3 = Clash switch

#DEFINE BANK0 BCF STATUS,5
#DEFINE BANK1 BSF STATUS,5

           include p16f628.inc

	   CBLOCK h'20'
COUNT
CLOCK
SPEED
	   ENDC

           __config  $3F30    ;activate the internal 4MHz oscillator

           ORG 0              ;
           goto START
           ORG 4              ;
           goto START
           ORG 5              ;

POWERON:   addwf PCL,F	      ;LED sequence for power up
	   retlw b'00000000'  ;
           retlw b'00000001'  ;
	   retlw b'00000011'  ;
	   retlw b'00000111'  ;
	   retlw b'00001111'  ;
           retlw b'00011111'  ;
	   retlw b'00111111'  ;
	   retlw b'01111111'  ;
	   retlw b'11111111'  ;

POWEROFF:  addwf PCL,F	      ;LED sequence for power down
	   retlw b'11111111'  ;
	   retlw b'01111111'  ;
	   retlw b'00111111'  ;
	   retlw b'00011111'  ;
           retlw b'00001111'  ;
	   retlw b'00000111'  ;
	   retlw b'00000011'  ;
	   retlw b'00000001'  ;
	   retlw b'00000000'  ;

START:     movlw 7	      ;configure Port A as a digital port
           movwf CMCON	      ;	
           clrf PORTA	      
           clrf PORTB	      
           BANK1		
           movlw B'00001100'  ;set Port A2 & A3 as inputs, all others are outputs
           movwf TRISA        ;
           movlw B'00000000'  ;set Port B0-B7 as outputs
           movwf PORTB
           movlw B'10000001'  ;TMR0 ratio = 1:4
           movwf OPTION_REG   ;bit 7 is high = pull-ups off
           BANK0

           movlw 25	      ;
	   movwf SPEED	      ;sets the led speed = 25 
           clrf PORTB	      ;all LEDs off

;test to see if the button is pressed to power up

GETKEY     btfss PORTA,2      ;is power up button pressed?
	   goto GETKEY	      ;no,loop around

           clrf COUNT	      ;clears count variable
                 
;LED power up routine

LOOP1      movf COUNT,W	      ;move count to w
	   call POWERON       ;get the led sequence
	   movwf PORTB        ;move the value to port b
	   call PAUSE         ;create a delay
           incf COUNT,F	      ;increment count variable
	   movf COUNT,W       ;check to see if count = 9
	   xorlw 9	      ;
	   btfsc STATUS,Z     ;
	   goto RESET         ;yes, go to reset
	   goto LOOP1         ;no, loop around

RESET      clrf COUNT	      ;clear count variable
           
;test to see if the button is still pressed
;and strobe the LEDs at the same time

GETKEY2    btfss PORTA,2      ;is the button still pressed?
           goto LOOP2	      ;no, go to power down routine
           movlw b'11111111'  ;
	   movwf PORTB	      ;yes, switch all LEDs on
           btfsc PORTA,3      ;is the clash button pressed?
           goto GETKEY2       ;yes, no need to strobe
	   movlw b'00000000'  ;
	   movwf PORTB        ;no, switch all LEDs off to strobe
	   goto GETKEY2       ;loop around

;LED power down routine

LOOP2      movf COUNT,W	      ;move count to w
	   call POWEROFF      ;get the led sequence
	   movwf PORTB        ;move the value to port b
	   call PAUSE         ;create a delay
           incf COUNT,F	      ;increment count variable
	   movf COUNT,W       ;check to see if count = 9
	   xorlw 9	      ;
	   btfsc STATUS,Z     ;
	   goto GETKEY        ;yes, go back to the start
	   goto LOOP2         ;no, loop around

;delay routine

PAUSE      movf SPEED,W       ;set pause counter to value of speed
           movwf CLOCK        ; 
           clrf INTCON        ;clear interupt flag
                             
LOOP4      btfss INTCON,2     ;has the intcon timer time-out been detected?
           goto LOOP4         ;no, loop until it is detected
           bcf INTCON,2       ;yes, clear intcon
           decfsz CLOCK,F     ;decrement the clock counter, is it zero?
           goto LOOP4         ;no
           return             ;yes
		
           END


