;LED SCANNER PROJECT - Version 1
;BRILLIANT LED PROJECTS - Nick Dossis - McGraw Hill 2011
;8 LEDs on Port B scan from one side to the other
;High signal on Port A,0 increases the speed of the LEDs
;High signal on Port A,1 decreases the speed of the LEDs

#DEFINE BANK0 BCF STATUS,5
#DEFINE BANK1 BSF STATUS,5

           include p16f628.inc

	   CBLOCK h'20'
CLOCK
SPEED
	   ENDC

           __config  $3F30    ;internal 4MHz oscillator

           ORG 0              ;
           goto START
           ORG 4              ;
           goto START
           ORG 5              ; 

START:     movlw 7	      ;configure port A as a digital port
           movwf CMCON	      ;	
           clrf PORTA	      
           clrf PORTB	      
           BANK1		
           movlw B'00000011'  ;set Port A0-A1 as inputs, all others are outputs
           movwf TRISA        ;
           movlw B'00000000'  ;set Port B0-B7 as outputs
           movwf PORTB
           movlw B'10000010'  ;TMR0 rate = 1:8
           movwf OPTION_REG   ;bit 7 high = pull-ups off
           BANK0

           movlw 30	      ;
	   movwf SPEED	      ;sets the led speed at 30 as a default
                 
LOOP1      movlw B'00000001'  ;move 1 to w
           movwf PORTB        ;send it to Port B to illuminate the first led
           bcf STATUS,C       ;clear the carry flag
                 
LOOP2      call PAUSE	      ;create a delay
	   call GETKEY 	      ;test to see if a key has been pressed
           rlf PORTB,F        ;rotate Port B output to move the led to the left
           btfss STATUS,C     ;if the carry flag is set then led B7 has been lit so skip a line
           goto LOOP2         ;loop around to continue moving the led

	   movlw B'10000000'  ;activate the led on Port B,7
           movwf PORTB        ;
           bcf STATUS,C       ;clear the carry flag

LOOP3      call PAUSE
	   call GETKEY
           rrf PORTB,F        ;rotate Port B right to move the led the opposite way
           btfss STATUS,C     ;if the carry flag is set then led B0 has been lit so skip a line
           goto LOOP3         ;loop around to continue moving the led
	
           goto LOOP1         ;start the sequence all over again

PAUSE      movf SPEED,W       ;move speed into the clock counter
           movwf CLOCK        ; 
           clrf INTCON        ;clear interrupt flag
                             
LOOP4      btfss INTCON,2     ;has the intcon timer time-out been detected?
           goto LOOP4         ;no, loop until it is detected
           bcf INTCON,2       ;yes, clear intcon
           decfsz CLOCK,F     ;decrement the clock counter, is it zero?
           goto LOOP4         ;no
           return             ;yes

GETKEY     movf PORTA,W	      ;moves the value of port a to w
	   xorlw B'00000000'  ;are any keys pressed?
	   btfsc STATUS,Z     ;
	   return	      ;no, return
	   btfsc PORTA,0      ;yes, is A,0 pressed?
	   incf SPEED,F	      ;yes, increment speed
           btfsc PORTA,1      ;is A,1 pressed?
	   decf SPEED,F	      ;yes, decrement speed
	   movf SPEED,W       ;check to see if speed = 61
	   xorlw 61	      ;
	   btfsc STATUS,Z     ;
	   goto RESET         ;yes, reset value of speed

	   movf SPEED,W       ;check to see if speed equals zero
	   xorlw 0	      ;
	   btfss STATUS,Z     ;
	   return	      ;
	   movlw 60	      ;if it is then reset the speed to 60
	   movwf SPEED	      ;
	   return	      ;
	
RESET	   movlw 1            ;reset the speed to 1
           movwf SPEED	
           return
		
           END


